@@ -19,7 +19,6 @@ import ipaddress
 from oslo_utils import uuidutils
 import re
 import six
-from six.moves.urllib import parse
 import string
 
 from manilaclient import api_versions
@@ -27,7 +26,6 @@ from manilaclient import base
 from manilaclient.common.apiclient import base as common_base
 from manilaclient.common import constants
 from manilaclient import exceptions
-from manilaclient import utils
 from manilaclient.v2 import share_instances
 
 
@@ -78,9 +76,10 @@ class Share(common_base.Resource):
         """Delete the specified share ignoring its current state."""
         self.manager.force_delete(self)
 
-    def allow(self, access_type, access, access_level):
+    def allow(self, access_type, access, access_level, metadata=None):
         """Allow access to a share."""
-        return self.manager.allow(self, access_type, access, access_level)
+        return self.manager.allow(
+            self, access_type, access, access_level, metadata)
 
     def deny(self, id):
         """Deny access from IP to a share."""
@@ -397,13 +396,7 @@ class ShareManager(base.ManagerWithFind):
             else:
                 search_opts['export_location_path'] = export_location
 
-        query_string = ""
-        if search_opts:
-            search_opts = utils.unicode_key_value_to_string(search_opts)
-            params = sorted(
-                [(k, v) for (k, v) in list(search_opts.items()) if v])
-            if params:
-                query_string = "?%s" % parse.urlencode(params)
+        query_string = self._build_query_string(search_opts)
 
         if detailed:
             path = "/shares/detail%s" % (query_string,)
@@ -445,14 +438,29 @@ class ShareManager(base.ManagerWithFind):
             exc_str = ('Invalid CN (common name). Must be 1-64 chars long.')
             raise exceptions.CommandError(exc_str)
 
+    '''
+    for the reference specification for AD usernames, reference below links:
+
+    1:https://docs.microsoft.com/en-us/previous-versions/windows/it-pro/
+            windows-server-2008-R2-and-2008/cc733146(v=ws.11)
+    2:https://docs.microsoft.com/en-us/previous-versions/windows/it-pro/
+            windows-server-2000/bb726984(v=technet.10)
+    '''
     @staticmethod
     def _validate_username(access):
-        valid_username_re = '[\w\$\.\-_\`;\'\{\}\[\]\\\\]{4,255}$'
+        sole_periods_spaces_re = '[\s|\.]+$'
+        valid_username_re = '.[^\"\/\\\[\]\:\;\|\=\,\+\*\?\<\>]{3,254}$'
         username = access
+
+        if re.match(sole_periods_spaces_re, username):
+            exc_str = ('Invalid user or group name,cannot consist solely '
+                       'of periods or spaces.')
+            raise exceptions.CommandError(exc_str)
+
         if not re.match(valid_username_re, username):
             exc_str = ('Invalid user or group name. Must be 4-255 characters '
                        'and consist of alphanumeric characters and '
-                       'special characters $]{.-_\'`;}[\\')
+                       'exclude special characters "/\[]:;|=,+*?<>')
             raise exceptions.CommandError(exc_str)
 
     @staticmethod
@@ -505,13 +513,15 @@ class ShareManager(base.ManagerWithFind):
                    ', '.join(valid_access_types))
             raise exceptions.CommandError(msg)
 
-    def _do_allow(self, share, access_type, access, access_level, action_name):
+    def _do_allow(self, share, access_type, access, access_level, action_name,
+                  metadata=None):
         """Allow access to a share.
 
         :param share: either share object or text with its ID.
         :param access_type: string that represents access type ('ip','domain')
         :param access: string that represents access ('127.0.0.1')
         :param access_level: string that represents access level ('rw', 'ro')
+        :param metadata: A dict of key/value pairs to be set
         """
         access_params = {
             'access_type': access_type,
@@ -519,37 +529,47 @@ class ShareManager(base.ManagerWithFind):
         }
         if access_level:
             access_params['access_level'] = access_level
+        if metadata:
+            access_params['metadata'] = metadata
         access = self._action(action_name, share,
                               access_params)[1]["access"]
         return access
 
     @api_versions.wraps("1.0", "2.6")
-    def allow(self, share, access_type, access, access_level):
+    def allow(self, share, access_type, access, access_level, metadata=None):
         self._validate_access(access_type, access)
         return self._do_allow(
             share, access_type, access, access_level, "os-allow_access")
 
     @api_versions.wraps("2.7", "2.12")  # noqa
-    def allow(self, share, access_type, access, access_level):
+    def allow(self, share, access_type, access, access_level, metadata=None):
         self._validate_access(access_type, access)
         return self._do_allow(
             share, access_type, access, access_level, "allow_access")
 
     @api_versions.wraps("2.13", "2.37")  # noqa
-    def allow(self, share, access_type, access, access_level):
+    def allow(self, share, access_type, access, access_level, metadata=None):
         valid_access_types = ('ip', 'user', 'cert', 'cephx')
         self._validate_access(access_type, access, valid_access_types)
         return self._do_allow(
             share, access_type, access, access_level, "allow_access")
 
-    @api_versions.wraps("2.38")  # noqa
-    def allow(self, share, access_type, access, access_level):
+    @api_versions.wraps("2.38", "2.44")  # noqa
+    def allow(self, share, access_type, access, access_level, metadata=None):
         valid_access_types = ('ip', 'user', 'cert', 'cephx')
         self._validate_access(access_type, access, valid_access_types,
                               enable_ipv6=True)
         return self._do_allow(
             share, access_type, access, access_level, "allow_access")
 
+    @api_versions.wraps("2.45")  # noqa
+    def allow(self, share, access_type, access, access_level, metadata=None):
+        valid_access_types = ('ip', 'user', 'cert', 'cephx')
+        self._validate_access(access_type, access, valid_access_types,
+                              enable_ipv6=True)
+        return self._do_allow(
+            share, access_type, access, access_level, "allow_access", metadata)
+
     def _do_deny(self, share, access_id, action_name):
         """Deny access to a share.
 
@@ -582,7 +602,7 @@ class ShareManager(base.ManagerWithFind):
     def access_list(self, share):
         return self._do_access_list(share, "os-access_list")
 
-    @api_versions.wraps("2.7")  # noqa
+    @api_versions.wraps("2.7", "2.44")  # noqa
     def access_list(self, share):
         return self._do_access_list(share, "access_list")
 
